/*
 *
 * Simple wait and delay routines using Timer 2
 * 1 ms 7-segment display multiplexer
 * Display primitives
 *
 * Juha Niinikoski 7.4.2003
 *
 * Project switched to PIC16F818 04.09.2003
 * TMR0 frequency corrected 05.09.2003
 *
 * Character generator for Mitsu hardware 31.01.2005
 *
*/

// delays calculated for 4 MHz clock

/* wait counters */

static near ms_counter;		// 1 second counter
static near int msw;		// ms wait counter


/* Character generator & display variables */

// define segment signals (Mitsu Hardware)

#define A	0x02
#define B	0x10
#define C	0x40
#define D	0x01
#define E	0x04
#define F	0x08
#define G	0x80
#define DP	0x20
//#define DP 0x10			// decimal point bit

// define 7-segment characters

#define C_0	(A+B+C+D+E+F)
#define C_1	(B+C)
#define C_2	(A+B+G+E+D)
#define C_3	(A+B+C+D+G)
#define C_4	(F+G+B+C)
#define C_5	(A+F+G+C+D)
#define C_6	(A+F+E+D+C+G)
#define C_7	(A+B+C)
#define C_8	(A+B+C+D+E+F+G)
#define C_9	(A+B+C+D+F+G)

#define C_NEG (G)
#define C_o (G+C+D+E)
#define C_u (C+D+E)
#define C_i (C)
#define C_n (E+G+C)


static near unsigned char digit_ctr;		// digit counter
//static near unsigned char digit_data[3];	// segment bits

const unsigned char chgen[32] =
{C_0, C_1, C_2, C_3, C_4, C_5, C_6, C_7, C_8, C_9, C_NEG, C_i, C_o,  C_NEG, DP, 0,
C_u, C_n, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,};

const unsigned char digit_bits[3] = {0x40, 0x04, 0x10};	// digit select bits


/* Init TMR2 for 1 mS interrupt generation */

void init_timers(void)
	{
	digit_ctr = 0;
	s0 = 0;
	ms_counter = 1000;	// init counters

	TMR2ON = 1;		// start TIMER2
	PR2 = 249;		// set TMR2 cycle time 1mS
	T2CKPS0 = 1;		// Prescaler = 4 = 250kHz clock for TMR2

	TMR2IE = 1;		// enable TMR2 IRQ
	PEIE = 1;		// enable peripheral IRQs
	GIE = 1;		// enable interrupts

	OPTION = (OPTION & 0xCF);  /* 128 prescale for wdt, 1MHz clock for TMR0 */
	}


/* Display limited int on LED display */

void putintdec( int c, char dp)
{
	unsigned char neg_sign;

	if (c > 999)			/* clip to max reading */
		c = 999;
	if (c < 0)			/* check if negative value */
		{
		neg_sign = 1;
		c = 0 - c;		/* invert it */
		if (c > 99)
			c = 99;		/* max negative reading */
		}
	else
		neg_sign = 0;

	//hundreds
	if (neg_sign == 1)
		digit_data[2] = chgen[10];  		// - sign
	else
		digit_data[2] = chgen[(c/100)];
	
	c-=(c/100)*100;

	if (dp == 2)
		digit_data[2] =	digit_data[2] | DP;	// set DP

	//tens
	digit_data[1] = chgen[(c/10)];
	c-=(c/10)*10;

	if (dp == 1)
		digit_data[1] =	digit_data[1] | DP;	// set DP

	//ones
	digit_data[0] = chgen[(c/1)];
}

/* Display therr byte long string on LED display */

void putst(register const char *str)
{
	digit_data[2] = chgen[(*str & 0x1F)]; 
	str++;
	digit_data[1] = chgen[(*str & 0x1F)]; 
	str++;
	digit_data[0] = chgen[(*str & 0x1F)]; 
}



/* Millisecond delays */

void delayms( int millisec )
	{
	msw = millisec; 
	while(msw != 0)
		;

	}
/* Delays a multiple microseconds + start delay at 4 MHz  TMR0 timer */

void delayus( unsigned char usec )
{
	TMR0 = 0;
	while ( TMR0 < usec );
}
	

/* IRQ handler */

static void interrupt tm02int()
{
	TMR2IF = 0;			// reset IRQ flag

// uppdate display every millisecond

	digit_ctr ++;				// uppdate digit counter
	if(digit_ctr > 2)
		digit_ctr = 0;

	SEGMENT_DRIVE = 0;			// shut display off
	DIGIT_DRIVE = digit_bits[digit_ctr];	// set new digit drive
	SEGMENT_DRIVE = digit_data[digit_ctr];	// set new segment bits / displat on

	if(msw != 0)			// decrement ms delay counter
		{
		msw--;
		}


	ms_counter --;			// divide seconds
	if(ms_counter == 0)		// secondoccured, do 1s tasks
		{
		ms_counter = 1000;
		s_flag = 1;		// second elapsed
		feed_dog = 1;		// signal to main prog: Time to feed Watchdog

		if(s0 != 0)		// decrement second wait counter
			{
			s0--;
			}
		}
}

